#include "dun.h"
#include "dunconst.h"
#include "dunrand1.h"
#include "hero.h"
#include "bg.h"

//---------------------------------------------------------------------------
ST_DUN Dun;


//---------------------------------------------------------------------------
EWRAM_CODE void DunInit()
{
	if(Dun.pTile != NULL)
	{
		DunFreeTile();
	}

	_Memset((u8*)&Dun, 0x00, sizeof(ST_DUN));
}
//---------------------------------------------------------------------------
EWRAM_CODE bool DunLoad()
{
	if(SramIsStr("CDBS") == FALSE)
	{
		return FALSE;
	}

	u16 size = SramRead16();
	u16 ver  = SramRead16();

	SramReadCopy((u8*)&Dun, size);

	Dun.pTile = MyMemCalloc(Dun.tileSize);
	SramReadCopy((u8*)&Dun.pTile, Dun.tileSize);

	return TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void DunSave()
{
	u16 size = sizeof(ST_DUN);
	u16 ver  = 100;

	SramWriteCopy((u8*)"CDBS", 4);
	SramWrite16(size);
	SramWrite16(ver);

	SramWriteCopy((u8*)&Dun, size);
	SramWriteCopy((u8*)&Dun.pTile, Dun.tileSize);
}
//---------------------------------------------------------------------------
EWRAM_CODE void DunDraw()
{
	s16 mx = HeroGetX() - 3;
	s16 my = HeroGetY() - 2;

	s16 x, y;
	u16 tileNo;

	for(y=0; y<5; y++)
	{
		for(x=0; x<8; x++)
		{
			tileNo = DunGetTileNo(mx+x, my+y);

			BgDrawTile32(x, y, tileNo);
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void DunMake(s16 floorCnt)
{
	ST_DUN* p = &Dun;

	if(p->pTile != NULL)
	{
		DunFreeTile();
	}

	// Œ}bv_}bv肵܂
	char buf[40];
	_Sprintf(buf, "f%d.txt", floorCnt);
	char* pStr = GBFSGetPointer(buf);

	if(pStr != NULL)
	{
		DunConstMake(pStr);
		Dun.id = DUN_ID_CONST;
	}
	else if(floorCnt >= 15)
	{
		// TODO DunRand3Make(DUN_CX, DUN_CY, floorCnt);
		DunRand1Make(DUN_CX, DUN_CY, floorCnt);
		Dun.id = DUN_ID_RAND3;
	}
	else if(floorCnt >= 7)
	{
		// TODO DunRand2Make(DUN_CX, DUN_CY, floorCnt);
		DunRand1Make(DUN_CX, DUN_CY, floorCnt);
		Dun.id = DUN_ID_RAND2;
	}
	else
	{
		DunRand1Make(DUN_CX, DUN_CY, floorCnt);
		Dun.id = DUN_ID_RAND1;
	}

	HeroSetXY(Dun.stepUpX, Dun.stepUpY);
}
//---------------------------------------------------------------------------
EWRAM_CODE void DunTurn(s16 floorCnt)
{
	switch(Dun.id)
	{
	case DUN_ID_CONST:
		DunConstTurn(floorCnt);
		break;

	case DUN_ID_RAND1:
		DunRand1Turn(floorCnt);
		break;

	case DUN_ID_RAND2:
		// TODO DunRand2Turn((floorCnt);
		break;

	case DUN_ID_RAND3:
		// TODO DunRand3Turn((floorCnt);
		break;

	default:
	case DUN_ID_NONE:
		SystemError("[Err] DunTurn Dun.id");
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void DunCallocTile(s16 cx, s16 cy)
{
	Dun.cx       = cx;
	Dun.cy       = cy;
	Dun.tileSize = cx * cy * sizeof(ST_TILE);
	Dun.pTile    = MyMemCalloc(Dun.tileSize);
	Dun.partCnt  = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void DunFreeTile()
{
	MyMemFree((u32)Dun.pTile, Dun.tileSize);
	Dun.pTile = NULL;
}
//---------------------------------------------------------------------------
EWRAM_CODE void DunSetMiniMap(s16 nx, s16 ny)
{
	s16 roomIdx = DunGetRoomIdx(nx, ny);

	if(roomIdx == -1)
	{
		// 
		DunSetMiniMapWay(nx, ny);
	}
	else
	{
		// 
		DunSetMiniMapRoom(roomIdx);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void DunSetMiniMapWay(s16 nx, s16 ny)
{
	s16 x, y;

	for(y=-1; y<2; y++)
	{
		for(x=-1; x<2; x++)
		{
 			ST_TILE* pTile = DunGetTile(nx+x, ny+y);

 			if(pTile != NULL)
 			{
 				pTile->isMiniMap = TRUE;
 			}
 		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void DunSetMiniMapRoom(s16 idx)
{
	ST_RECT* p = &Dun.part[idx].room;
	s16 x, y;

	for(y=p->top; y<p->bottom; y++)
	{
		for(x=p->left; x<p->right; x++)
		{
 			ST_TILE* pTile = DunGetTile(x, y);

 			if(pTile != NULL)
 			{
 				pTile->isMiniMap = TRUE;
 			}
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE s16 DunGetCx()
{
	return Dun.cx;
}
//---------------------------------------------------------------------------
EWRAM_CODE s16 DunGetCy()
{
	return Dun.cy;
}
//---------------------------------------------------------------------------
EWRAM_CODE s16 DunGetRoomIdx(s16 x, s16 y)
{
	s16 i;

	for(i=0; i<Dun.partCnt; i++)
	{
		ST_RECT* p = &Dun.part[i].room;

		if(p->top > y || p->bottom < y)
		{
			continue;
		}
		if(p->left > x || p->right < x)
		{
			continue;
		}

		return i;
	}

	return -1;
}
//---------------------------------------------------------------------------
EWRAM_CODE s16 DunGetRectIdx(s16 x, s16 y)
{
	s16 i;

	for(i=0; i<Dun.partCnt; i++)
	{
		ST_RECT* p = &Dun.part[i].rect;

		if(p->top > y || p->bottom < y)
		{
			continue;
		}
		if(p->left > x || p->right < x)
		{
			continue;
		}

		return i;
	}

	return -1;
}
//---------------------------------------------------------------------------
EWRAM_CODE s16 DunGetPartCnt()
{
	return Dun.partCnt;
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 DunGetTileNo(s16 x, s16 y)
{
	ST_TILE* pTile = DunGetTile(x, y);

	if(pTile == NULL)
	{
		return TILE_BLACK;
	}

	if(pTile->isWall == TRUE)
	{
		ST_TILE* pTile2 = DunGetTile(x, y+1);

		if(pTile2 != NULL && pTile2->isWall == FALSE)
		{
			return TILE_WALL_UP;
		}

		return TILE_WALL;
	}

	if(pTile->isStepUp == TRUE)
	{
		return TILE_EARTH_STEP;
	}

	if(pTile->isStepDown == TRUE)
	{
		return TILE_BLACK_STEP;
	}

	if(pTile->isTrapShow == TRUE)
	{
		return TILE_TRAP;
	}

	if(pTile->item.id != ITEM_ID_NONE)
	{
		return TILE_ITEM;
	}

	return TILE_EARTH;
}
//---------------------------------------------------------------------------
EWRAM_CODE ST_TILE* DunGetTile(s16 x, s16 y)
{
	if(x < 0 || y < 0 || x >= Dun.cx || y >= Dun.cy)
	{
		return NULL;
	}

	return &Dun.pTile[y*Dun.cx + x];
}
//---------------------------------------------------------------------------
EWRAM_CODE ST_RECT* DunGetRoom(s16 idx)
{
	return &Dun.part[idx].room;
}
//---------------------------------------------------------------------------
EWRAM_CODE void DunDebug()
{
	s16 x, y;

	TRACEOUT("--DunDebug Start--\n");

	for(y=0; y<Dun.cy; y++)
	{
		for(x=0; x<Dun.cx; x++)
		{
			ST_TILE* p = DunGetTile(x, y);

			if(p->isWall == TRUE)
			{
				TRACEOUT("#");
			}
			else if(p->item.id != ITEM_ID_NONE)
			{
				TRACEOUT("+");
			}
			else
			{
				TRACEOUT("-");
			}
		}
		TRACEOUT("\n");
	}

	TRACEOUT("--DunDebug End--\n");
}
