#include "game.h"
#include "hero.h"
#include "key.h"
#include "msg.h"
#include "fade.h"
#include "mobmap.h"

//---------------------------------------------------------------------------
ST_GAME Game;

// key.c
extern ST_KEY Key;

//---------------------------------------------------------------------------
EWRAM_CODE void GameInit()
{
	_Memset(&Game, 0x00, sizeof(ST_GAME));

	MsgInit();
	SelInit();
	DirInit();
	HeroInit();

	GameActNextFloor();
}
//---------------------------------------------------------------------------
EWRAM_CODE bool GameLoad()
{
	if(SramIsStr("CGAME") == FALSE)
	{
		return TRUE;
	}

	u16 size = SramRead16();
	u16 ver  = SramRead16();

	SramReadCopy((u8*)&Game, size);

	return FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameSave()
{
	u16 size = sizeof(ST_GAME);
	u16 ver  = 100;

	SramWriteCopy((u8*)"CGAME", 5);
	SramWrite16(size);
	SramWrite16(ver);

	SramWriteCopy((u8*)&Game, size);
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameDraw()
{
	DunDraw();
	HeroDraw();
	MobDraw();
	MsgDraw();
	SelDraw();
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameTurn()
{
	HeroTurn();
	MobTurn();
	DunTurn(Game.floorCnt);
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameExec()
{
	DirExec();


	// ----
	GameExecChk();

	switch(Game.id)
	{
	case EXEC_ID_NONE:  /* EMPTY */      break;
	case EXEC_ID_MOVE:  GameExecMove();  break;			// ړ
	case EXEC_ID_EQUIP: GameExecEquip(); break;			// ACe𑕔
	case EXEC_ID_GET:   GameExecGet();   break;			// ACeE
	case EXEC_ID_DROP:  GameExecDrop();  break;			// ACêĂ
	case EXEC_ID_USE:   GameExecUse();   break;			// ACeg
	case EXEC_ID_MAGIC: GameExecMagic(); break;			// @
	case EXEC_ID_ARROW: GameExecArrow(); break;			// 
	case EXEC_ID_OVER:  GameExecOver();  break;			// Q[I[o[
	case EXEC_ID_PROVE: GameExecProve(); break;			// Ӓ̊mF
	case EXEC_ID_STEP:  GameExecStep();  break;			// Ki̊mF
	case EXEC_ID_SAVE:  GameExecSave();  break;			// Z[u
	case EXEC_ID_LOAD:  GameExecLoad();  break;			// [h

	default:
		SystemError("[Err] GameExec Game,execId");
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameExecChk()
{
	if(Game.isHold == TRUE)
	{
		return;
	}

	if(HeroIsDead() == TRUE)
	{
		GameSetExec(EXEC_ID_OVER, FALSE);
		return;
	}

	// L[͂̃`FbN܂
	GameSetExec(EXEC_ID_NONE, FALSE);

	if(Key.rep & DPAD)
	{
		GameSetExec(EXEC_ID_MOVE, FALSE);
	}
	else if(Key.rep & KEY_A)
	{
		GameSetExec(EXEC_ID_USE, FALSE);
	}
	else if(Key.rep & KEY_B)
	{
		GameSetExec(EXEC_ID_GET, FALSE);
	}
	else if(Key.rep & KEY_L)
	{
		GameSetExec(EXEC_ID_EQUIP, FALSE);
	}
	else if(Key.rep & KEY_R)
	{
		GameSetExec(EXEC_ID_DROP, FALSE);
	}
	else if(Key.rep & KEY_START)
	{
		GameSetExec(EXEC_ID_SAVE, FALSE);
	}
	else if(Key.rep & KEY_SELECT)
	{
		GameSetExec(EXEC_ID_LOAD, FALSE);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameExecOver()
{
	Game.isOver = TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameExecMove()
{
	s16 mx = 0;
	s16 my = 0;

	if(Key.rep & KEY_UP)    my = -1;
	if(Key.rep & KEY_DOWN)  my =  1;
	if(Key.rep & KEY_LEFT)  mx = -1;
	if(Key.rep & KEY_RIGHT) mx =  1;

	s16 nx = HeroGetX() + mx;
	s16 ny = HeroGetY() + my;


	// X^[
	ST_MOB_CHR* pMob = MobGetChr(nx, ny);

	if(pMob != NULL)
	{
		HeroDir(nx, ny);

		GameActHeroAtk(pMob);
		GameTurn();
		return;
	}


	// ^C
	ST_TILE* pTile = DunGetTile(nx, ny);

	if(pTile == NULL || pTile->isWall == TRUE)
	{
		return;
	}

	HeroMove(nx, ny);
	GameTurn();

	// ACe
	if(pTile->item.id != ITEM_ID_NONE)
	{
		MsgAdd(MSG_ID_INFO, "%sĂ", ItemGetName(&pTile->item));
	}

	// 
	if(pTile->trapId != TRAP_ID_NONE)
	{
		GameActTrap(pTile);
	}

	// Ki
	if(pTile->isStepDown == TRUE)
	{
		GameSetExec(EXEC_ID_STEP, TRUE);
	}

	// ~j}bv
	DunSetMiniMapWay(nx, ny);

}
//---------------------------------------------------------------------------
EWRAM_CODE void GameExecStep()
{
	// param0: State

	switch(Game.param[0])
	{
	// INIT
	case 0:
		MsgAdd(MSG_ID_INFO, "̊KɈړ܂H");

		Game.isHold = TRUE;
		Game.param[0] = 1;
		break;

	// SELECT
	case 1:
		if(Key.rep & KEY_A)
		{
			FadeIn();
			GameActNextFloor();
			FadeOut();

			MsgAdd(MSG_ID_INFO, "tA%dɓ", Game.floorCnt);
			Game.param[0] = 2;
		}
		else if(Key.rep & KEY_B)
		{
			MsgAdd(MSG_ID_INFO, "ړ̂߂");
			Game.param[0] = 2;
		}
		break;

	// END
	case 2:
		Game.isHold = FALSE;
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameExecEquip()
{
	// param0: State
	// param1: HeroItemCnt
	// param2: selectCnt

	u16 maskId = ITEM_ID_WEAPON | ITEM_ID_ARMOR | ITEM_ID_RING;

	switch(Game.param[0])
	{
	// INIT
	case 0: ;
		s16 cnt = HeroGetItemCnt2(maskId);

		if(cnt == 0)
		{
			MsgAdd(MSG_ID_INFO, "ACe͎ĂȂ");
			return;
		}
		MsgAdd(MSG_ID_INFO, "ACeIł");

		Game.isHold = TRUE;
		Game.param[0] = 1;
		Game.param[1] = cnt;
		break;

	// DRAW ITEM
	case 1: ;
		SelAdd(HeroGetItemStr2(Game.param[2], maskId));
		Game.param[0] = 2;
		break;

	// SELECT ITEM
	case 2:
		if(Key.rep & KEY_UP)
		{
			if(Game.param[2] < Game.param[1])
			{
				Game.param[2]++;
				Game.param[0] = 1;
			}
		}
		else if(Key.rep & KEY_DOWN)
		{
			if(Game.param[2] > 0)
			{
				Game.param[2]--;
				Game.param[0] = 1;
			}
		}
		else if(Key.rep & KEY_B)
		{
			MsgAdd(MSG_ID_INFO, "ύX߂");
			Game.param[0] = 3;
		}
		else if(Key.rep & KEY_A)
		{
			ST_ITEM* pItem2 = HeroGetItem2(Game.param[2], maskId);

			MsgAdd(MSG_ID_INFO, "u%sv𑕔", ItemGetName(pItem2));
			HeroSetEquip(pItem2);
			Game.param[0] = 3;
		}
		break;

	// END
	case 3:
		SelHide();
		Game.isHold = FALSE;
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameExecGet()
{
	s16 x = HeroGetX();
	s16 y = HeroGetY();
	ST_TILE* pTile = DunGetTile(x, y);

	if(pTile->item.id == ITEM_ID_NONE)
	{
		MsgAdd(MSG_ID_INFO, "Ê͂Ȃ");
		return;
	}

	if(HeroIsItemFull() == TRUE)
	{
		MsgAdd(MSG_ID_INFO, "ACeς");
		return;
	}

	MsgAdd(MSG_ID_INFO, "u%sv肵", ItemGetName(&pTile->item));
	HeroAddItem(&pTile->item);

	pTile->item.id = ITEM_ID_NONE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameExecDrop()
{
	// param0: State
	// param1: HeroItemCnt
	// param2: selectCnt

	switch(Game.param[0])
	{
	// INIT
	case 0: ;
		s16 cnt = HeroGetItemCnt();

		if(cnt == 0)
		{
			MsgAdd(MSG_ID_INFO, "̂ĂACeĂȂ");
			return;
		}
		MsgAdd(MSG_ID_INFO, "̂ĂACeIł");

		Game.isHold = TRUE;
		Game.param[0] = 1;
		Game.param[1] = cnt;
		break;

	// DRAW ITEM
	case 1: ;
		SelAdd(HeroGetItemStr(Game.param[2]));
		Game.param[0] = 2;
		break;

	// SELECT ITEM
	case 2:
		if(Key.rep & KEY_UP)
		{
			if(Game.param[2] < Game.param[1])
			{
				Game.param[2]++;
				Game.param[0] = 1;
			}
		}
		else if(Key.rep & KEY_DOWN)
		{
			if(Game.param[2] > 0)
			{
				Game.param[2]--;
				Game.param[0] = 1;
			}
		}
		else if(Key.rep & KEY_B)
		{
			MsgAdd(MSG_ID_INFO, "̂Ă̂߂");
			Game.param[0] = 3;
		}
		else if(Key.rep & KEY_A)
		{
			ST_ITEM* pItem2 = HeroGetItem(Game.param[2]);
			MsgAdd(MSG_ID_INFO, "u%sv̂Ă", ItemGetName(pItem2));

			HeroDelItem(Game.param[2]);
			Game.param[0] = 3;
		}
		break;

	// END
	case 3:
		SelHide();
		Game.isHold = FALSE;
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameExecUse()
{
	// param0: State
	// param1: HeroItemCnt
	// param2: selectCnt

	u16 maskId = ITEM_ID_USEITEM;

	switch(Game.param[0])
	{
	// INIT
	case 0: ;
		s16 cnt = HeroGetItemCnt2(maskId);

		if(cnt == 0)
		{
			MsgAdd(MSG_ID_INFO, "gACeĂȂ");
			return;
		}
		MsgAdd(MSG_ID_INFO, "gACeIł");

		Game.isHold     = TRUE;
		Game.param[0] = 1;
		Game.param[1] = cnt;
		break;

	// DRAW ITEM
	case 1: ;
		SelAdd(HeroGetItemStr2(Game.param[2], maskId));
		Game.param[0] = 2;
		break;

	// SELECT ITEM
	case 2:
		if(Key.rep & KEY_UP)
		{
			if(Game.param[2] < Game.param[1])
			{
				Game.param[2]++;
				Game.param[0] = 1;
			}
		}
		else if(Key.rep & KEY_DOWN)
		{
			if(Game.param[2] > 0)
			{
				Game.param[2]--;
				Game.param[0] = 1;
			}
		}
		else if(Key.rep & KEY_B)
		{
			MsgAdd(MSG_ID_INFO, "ĝ߂");
			Game.param[0] = 3;
		}
		else if(Key.rep & KEY_A)
		{
			if(GameActUse(Game.param[2]) == TRUE)
			{
				GameTurn();
				Game.param[0] = 3;
			}
			else
			{
				Game.param[0] = 4;
			}
		}
		break;

	// END
	case 3:
		SelHide();
		Game.isHold = FALSE;
		break;

	// END2
	case 4:
		// ɏԑJڂ܂
		SelHide();
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameExecMagic()
{
	// param0: State
	// param1: itemCnt(O)

	switch(Game.param[0])
	{
	// INIT
	case 0:
		SelAdd(MSG_ID_INFO, "˂肵Ă");
		Game.isHold = TRUE;
		Game.param[0] = 1;
		break;

	// SELECT ITEM
	case 1:
		if(Key.rep & DPAD)
		{
			GameActMagic(Game.param[1], Key.rep);
			GameTurn();
		}
		else if(Key.rep & KEY_B)
		{
			MsgAdd(MSG_ID_INFO, "gp̂߂");
		}

		Game.param[0] = 2;
		break;

	// END
	case 2:
		SelHide();
		Game.isHold = FALSE;
		break;
	}
}
//---------------------------------------------------------------------------
// ACeꗗĂяo܂
EWRAM_CODE void GameExecArrow()
{
	// param0: State
	// param1: itemCnt(O)

	switch(Game.param[0])
	{
	// INIT
	case 0:
		SelAdd(MSG_ID_INFO, "˂肵Ă");
		Game.isHold     = TRUE;
		Game.param[0] = 1;
		break;

	// SELECT ITEM
	case 1:
		if(Key.rep & DPAD)
		{
			GameActMagic(Game.param[1], Key.rep);
			GameTurn();
		}
		else if(Key.rep & KEY_B)
		{
			MsgAdd(MSG_ID_INFO, "gp̂߂");
		}

		Game.param[0] = 2;
		break;

	// END
	case 2:
		SelHide();
		Game.isHold = FALSE;
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameExecProve()
{
	// param0: State
	// param1: HeroItemCnt
	// param2: selectCnt
	// param3: unidentfiedNum

	u16 maskId = ITEM_ID_UNIDENTIFIED;

	switch(Game.param[0])
	{
	// INIT
	case 0: ;
		s16 cnt = HeroGetItemCnt2(maskId);

		if(cnt == 0)
		{
			MsgAdd(MSG_ID_INFO, "ӒACeĂȂ");
			return;
		}
		MsgAdd(MSG_ID_INFO, "Ӓ肷ACeIł");

		Game.isHold = TRUE;
		Game.param[0] = 1;
		Game.param[1] = HeroGetItemCnt();
		break;

	// DRAW ITEM
	case 1: ;
		SelAdd(HeroGetItemStr2(Game.param[2], maskId));
		Game.param[0] = 2;
		break;

	// SELECT ITEM
	case 2:
		if(Key.rep & KEY_UP)
		{
			if(Game.param[2] < Game.param[1])
			{
				Game.param[2]++;
				Game.param[0] = 1;
			}
		}
		else if(Key.rep & KEY_DOWN)
		{
			if(Game.param[2] > 0)
			{
				Game.param[2]--;
				Game.param[0] = 1;
			}
		}
		else if(Key.rep & KEY_B)
		{
			MsgAdd(MSG_ID_INFO, "̂Ă̂߂");
			Game.param[0] = 3;
		}
		else if(Key.rep & KEY_A)
		{
			ST_ITEM* pItem2 = HeroGetItem(Game.param[2]);
			pItem2->isUnidentified = FALSE;

			MsgAdd(MSG_ID_INFO, "u%svB", ItemGetName(pItem2));
			HeroDelItem2(Game.param[2], maskId);

			Game.param[0] = 3;
		}
		break;

	// END
	case 3:
		SelHide();
		Game.isHold = FALSE;
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameExecSave()
{
	
}
//---------------------------------------------------------------------------
EWRAM_CODE bool GameExecLoad()
{
	return FALSE;
}




//---------------------------------------------------------------------------
EWRAM_CODE void GameActHeroAtk(ST_MOB_CHR* pMob)
{
	s16 atk = HeroGetAtk();
	s16 dmg = MobAddDmg(pMob, atk);

	MsgAdd(MSG_ID_ATTACK, "%sɁA%d̃_[W^",  pMob->name, dmg);

	if(MobIsDead(pMob) == TRUE)
	{
		if(pMob->ai == 4)
		{
			MsgAdd(MSG_ID_INFO, "{XގI" );
			GameSetExec(EXEC_ID_OVER, TRUE);
			return;
		}

		MsgAdd(MSG_ID_INFO, "%sގ", pMob->name);
		HeroAddExp(pMob->exp);
		MobSort();
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameActMobAtk(ST_MOB_CHR* pMob)
{
	s16 atk = MobGetAtk(pMob);
	s16 dmg = HeroAddDmg(atk);

	MsgAdd(MSG_ID_ATTACK, "%d̃_[W", dmg);

	if(HeroIsDead() == TRUE)
	{
		MsgAdd(MSG_ID_INFO, "%sɂꂽI", pMob->name);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameActNextFloor()
{
	MobInit();
	DunInit();

	Game.floorCnt++;
	DunMake(Game.floorCnt);

//	DunDebug();
//	MobDebug();

	HeroSetNextFloor();
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameActTrap(ST_TILE* pTile)
{
	if(pTile->isTrapShow == TRUE)
	{
		return;
	}
	pTile->isTrapShow = TRUE;

	if(_Rand(4) != 0)
	{
		MsgAdd(MSG_ID_INFO, "iI");
		return;
	}

	switch(pTile->trapId)
	{
	case TRAP_ID_DAMAGE:
		if(_Rand(4) == 0)
		{
			s16 hp = HeroGetHp();
			s16 dmg = Div(hp + 3, 5);

			MsgAdd(MSG_ID_INFO, "iI%d̃_[W", dmg);
			HeroAddDmg(dmg);
		}
		else
		{
			MsgAdd(MSG_ID_INFO, "iI");
		}
		break;

	case TRAP_ID_POISON:
		MsgAdd(MSG_ID_INFO, "iIŏԂɂȂII" );
		HeroAddPoison(5);
		break;

	default:
		SystemError("GameActTrap pTile->trapId");
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameActMagic(s16 num, u16 key)
{
	ST_ITEM* pItem = HeroGetItem2(num, ITEM_ID_USEITEM);

	s16 mx = HeroGetX();
	s16 my = HeroGetY();
	s16 vx = 0;
	s16 vy = 0;

	if(key & KEY_UP)    vy--;
	if(key & KEY_DOWN)  vy++;
	if(key & KEY_LEFT)  vx--;
	if(key & KEY_RIGHT) vx++;


	s16 hitCnt = 0;

	for(;;)
	{
		mx += vx;
		my += vy;

		ST_TILE* pTile = DunGetTile(mx, my);
		if(pTile->isWall == TRUE)
		{
			break;
		}

		ST_MOB_CHR* pMob = MobGetChr(mx, my);
		if(pMob == NULL)
		{
			continue;
		}

		MsgAdd(MSG_ID_ATTACK, "%s̎gpɂ%sɖI", ItemGetName(pItem), pMob->name);

		s16 atk = HeroGetAtkMagic(pItem);
		MobAddDmg(pMob, atk);

		if(MobIsDead(pMob) == TRUE)
		{
			MsgAdd(MSG_ID_INFO, "%sގ", pMob->name);
			HeroAddExp(pMob->exp);
		}

		if(pItem->param[3] == MAGIC_ID_ONCE)
		{
			break;
		}
	}

	if(hitCnt == 0)
	{
		MsgAdd(MSG_ID_INFO, "Ȃ");
	}
	else
	{
		HeroDelItem(num);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void GameActArrow(s16 num, u16 key)
{
	ST_ITEM* pItem = HeroGetItem2(num, ITEM_ID_USEITEM);

	s16 mx = HeroGetX();
	s16 my = HeroGetY();
	s16 vx = 0;
	s16 vy = 0;

	if(key & KEY_UP)    vy--;
	if(key & KEY_DOWN)  vy++;
	if(key & KEY_LEFT)  vx--;
	if(key & KEY_RIGHT) vx++;


	bool isHit = 0;

	for(;;)
	{
		mx += vx;
		my += vy;

		ST_TILE* pTile = DunGetTile(mx, my);
		if(pTile->isWall == TRUE)
		{
			break;
		}

		ST_MOB_CHR* pMob = MobGetChr(mx, my);
		if(pMob == NULL)
		{
			continue;
		}

		MsgAdd(MSG_ID_ATTACK, "%s%sɖI", ItemGetName(pItem), pMob->name);

		s16 atk = HeroGetAtkArrow(pItem);
		MobAddDmg(pMob, atk);

		if(MobIsDead(pMob) == TRUE)
		{
			MsgAdd(MSG_ID_INFO, "%sގ", pMob->name);
			HeroAddExp(pMob->exp);
		}
	}

	if(isHit == 0)
	{
		MsgAdd(MSG_ID_INFO, "Ȃ");
	}
	else
	{
		// e܂
		pItem->param[2]--;

		if(pItem->param[2] == 0)
		{
			HeroDelItem2(num, ITEM_ID_USEITEM);
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE bool GameActUse(s16 cnt)
{
	ST_ITEM* pItem = HeroGetItem2(cnt, ITEM_ID_USEITEM);
	bool isUsed = FALSE;


	switch(pItem->param[0])
	{
	// HP
	case USEITEM_ID_HPCURE:
		isUsed = GameActUseHP(pItem);
		break;

	// MP
	case USEITEM_ID_MPCURE:
		isUsed = GameActUseHP(pItem);
		break;

	// 
	case USEITEM_ID_POISONCURE:
		isUsed = GameActUsePoison(pItem);
		break;

	// H
	case USEITEM_ID_FOOD:
		isUsed = GameActUseFood(pItem); 
		break;

	// [v
	case USEITEM_ID_WARP:
		isUsed = GameActUseWarp();
		break;

	// 
	case USEITEM_ID_ARROW:
		GameSetExec(EXEC_ID_ARROW, TRUE);
		Game.param[1] = cnt;
		break;

	// ACeӒ
	case USEITEM_ID_PROVE:
		GameSetExec(EXEC_ID_PROVE, TRUE);
		Game.param[1] = cnt;
		break;

	// @()
	case USEITEM_ID_MAGIC:
		switch(pItem->param[3])
		{
		// ^Cv(0)
		// ђʃ^Cv(1)
		case MAGIC_ID_ONCE:
		case MAGIC_ID_PENETRATE:
			GameSetExec(EXEC_ID_MAGIC, TRUE);
			Game.param[1] = cnt;
			break;

		// S̃^Cv
		case MAGIC_ID_ROOM:
			isUsed = GameActUseMagicRoom(pItem);
			break;

		default:
			SystemError("[Err] GameExecUse pItem->param[3]");
		}
		break;

	default:
		SystemError("[Err] GameExecUse pItem->param[0]");
		break;
	}

	if(isUsed == TRUE)
	{
		HeroDelItem(cnt);
	}

	return isUsed;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool GameActUseHP(ST_ITEM* pItem)
{
	s16 value = HeroAddHp(pItem->param[1]);

	if(value < 0)
	{
		MsgAdd(MSG_ID_NEGATIVE, "go%d", -value);
	}
	else
	{
		MsgAdd(MSG_ID_INFO, "go%d񕜂", value);
	}

	return TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool GameActUseMP(ST_ITEM* pItem)
{
	s16 value = HeroAddMp(pItem->param[1]);

	if(value < 0)
	{
		MsgAdd(MSG_ID_NEGATIVE, "lo%d", -value);
	}
	else
	{
		MsgAdd(MSG_ID_INFO, "lo%d񕜂", value);
	}

	return TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool GameActUsePoison(ST_ITEM* pItem)
{
	s16 value = pItem->param[1];
	s16 old   = HeroAddPoison(value);	// ł̗L^[Zbg(㏑)

	if( value != 0 )
	{
		MsgAdd(MSG_ID_NEGATIVE, "łI");
	}
	else
	{
		if(old != 0)
		{
			MsgAdd(MSG_ID_INFO, "ł񕜂");
		}
		else
		{
			// ŏԂłȂ̂Ɏgp
			MsgAdd(MSG_ID_INFO, "NȂ");
		}
	}
	return TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool GameActUseFood(ST_ITEM* pItem)
{
	HeroAddEat(pItem->param[1]);
	MsgAdd(MSG_ID_NEGATIVE, "H");

	return TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool GameActUseWarp()
{
	s16 partCnt = DunGetPartCnt();
	if(partCnt == 0)
	{
		goto End;
	}


	s16 i;

	// 5قǃ[vĂ݂܂
	for(i=0; i<5; i++)
	{
		ST_RECT* pRoom = DunGetRoom(_Rand(partCnt));

		s16 x = pRoom->left + _Rand(RECT_W(*pRoom));
		s16 y = pRoom->top  + _Rand(RECT_H(*pRoom));

		if(MobGetChr(x, y) == NULL)
		{
			HeroSetXY(x, y);
			MsgAdd(MSG_ID_INFO, "e|[gsI");

			return TRUE;
		}
	}

End:
	MsgAdd(MSG_ID_NEGATIVE, "ȂɂȂ");
	return FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool GameActUseMagicRoom(ST_ITEM* pItem)
{
	s16 cnt = MobGetCnt();

	if(cnt == 0)
	{
		MsgAdd(MSG_ID_INFO, "ȂɂȂA͏Ă܂");
		return TRUE;
	}

	s16 i;

	// SX^[Ƀ_[W^܂
	for(i=0; i<cnt; i++)
	{
		ST_MOB_CHR* pMob = MobGetChr2(i);
		s16 atk = HeroGetAtkMagic(pItem);

		MsgAdd(MSG_ID_INFO, "%s̎gpɂ%sɖI", ItemGetName(pItem), pMob->name);
		MobAddDmg(pMob, atk);

		if(MobIsDead(pMob) == TRUE)
		{
			HeroAddExp(pMob->exp);
		}
	}

	return TRUE;
}





//---------------------------------------------------------------------------
EWRAM_CODE void GameSetExec(u16 id, bool isHold)
{
	Game.id     = id;
	Game.isHold = isHold;

	s16 i;
	for(i=0; i<PARAM_MAX_CNT; i++)
	{
		Game.param[i] = 0;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE bool GameIsOver()
{
	return Game.isOver;
}
