#include "lex.h"

//---------------------------------------------------------------------------
ST_LEX Lex[LEX_MAX_CNT];


//---------------------------------------------------------------------------
EWRAM_CODE void LexInit()
{
	_Memset(&Lex, 0x00, sizeof(ST_LEX) * LEX_MAX_CNT);
}
//---------------------------------------------------------------------------
EWRAM_CODE s16 LexOpen(char* pCur)
{
	s16 i;

	for(i=0; i<LEX_MAX_CNT; i++)
	{
		if(Lex[i].isUse == FALSE)
		{
			Lex[i].isUse = TRUE;
			Lex[i].pCur  = pCur;
			Lex[i].pOld  = pCur;

			return i;
		}
	}

	return -1;
}
//---------------------------------------------------------------------------
EWRAM_CODE void LexClose(u16 fd)
{
	Lex[fd].isUse = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE s32 LexGetNum(u16 fd)
{
	LexGetToken(fd, FALSE);

	return (Lex[fd].id == TOKEN_NUM) ? Lex[fd].num : -1;
}
//---------------------------------------------------------------------------
EWRAM_CODE char* LexGetStr(u16 fd)
{
	LexGetToken(fd, FALSE);

	return (Lex[fd].id == TOKEN_STR) ? Lex[fd].str : NULL;
}
//---------------------------------------------------------------------------
EWRAM_CODE char LexGetChr(u16 fd)
{
	LexGetToken(fd, TRUE);

	return (Lex[fd].id == TOKEN_STR) ? Lex[fd].str[0] : '\0';
}
//---------------------------------------------------------------------------
EWRAM_CODE s32 LexGetSafeNum(u16 fd)
{
	s32 ret = LexGetNum(fd);

	if(ret == -1)
	{
		SystemError("[Err] LexGetNumSafe -1");
	}

	return ret;
}
//---------------------------------------------------------------------------
EWRAM_CODE char* LexGetSafeStr(u16 fd)
{
	char* ret = LexGetStr(fd);

	if(ret == NULL)
	{
		SystemError("[Err] LexGetStrSafe NULL");
	}

	return ret;
}
//---------------------------------------------------------------------------
EWRAM_CODE char LexGetSafeChr(u16 fd)
{
	char ret = LexGetChr(fd);

	if(ret == '\0')
	{
		SystemError("[Err] LexGetChrSafe \\0");
	}

	return ret;
}
//---------------------------------------------------------------------------
EWRAM_CODE char LexGetChrNoSeek(u16 fd)
{
	char ret = *Lex[fd].pCur++;

	return ret;
}
//---------------------------------------------------------------------------
EWRAM_CODE void LexTestChrSafe(u16 fd, char chr)
{
	char ret = LexGetChr(fd);

	if(ret != chr)
	{
		SystemError("[Err] LexTestSafeChr %c", chr);
	}
}






//---------------------------------------------------------------------------
EWRAM_CODE void LexGetToken(u16 fd, bool isChr)
{
	char buf[LEX_MAX_STR_CNT];
	s32  num, cnt;

	ST_LEX* p = &Lex[fd];
	p->pOld = p->pCur;

st: ;

	u16 id = LexGetId(*p->pCur);

	switch(id)
	{
	// XLbvi󔒁A^uj
	// R}
	case TOKEN_SKIP:
	case TOKEN_CUM:
		p->pCur++;
		goto st;

	// CR
	case TOKEN_CR:
		p->pCur += 2;
		goto st;

	// LF
	case TOKEN_LF:
		p->pCur++;
		goto st;

	// I[
	case TOKEN_END:
		p->id = id;
		break;

	// 
	case TOKEN_NUM:
		num = 0;
		cnt = 0;

		do {
			num = (num * 10) + (*p->pCur++ - '0');
			cnt++;

		} while((LexGetId(*p->pCur) == TOKEN_NUM) && (cnt < 5));

		p->id  = id;
		p->num = num;
		break;

	// 
	case TOKEN_STR:
		cnt = 0;

		do {
			buf[cnt++] = *p->pCur++;

		} while((LexGetId(*p->pCur) == TOKEN_STR) && (cnt < LEX_MAX_STR_CNT-1) && (isChr == FALSE));

		buf[cnt] = '\0';

		p->id  = id;
		p->num = cnt;
		_Strncpy(p->str, buf, cnt+1);
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 LexGetId(char chr)
{
	if(chr == ' ' || chr == '\t')
	{
		return TOKEN_SKIP;
	}
	else if(chr == CR)
	{
		return TOKEN_CR;
	}
	else if(chr == LF)
	{
		return TOKEN_LF;
	}
	else if(chr == ',')
	{
		return TOKEN_CUM;
	}
	else if(chr == '\0')
	{
		return TOKEN_END;
	}
	else if(_IsDigit(chr) == TRUE)
	{
		return TOKEN_NUM;
	}

	return TOKEN_STR;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool LexSeekTag(u16 fd, char* str)
{
	ST_LEX* p = &Lex[fd];

	while(*p->pCur != '\0')
	{
		if(_Strncmp(p->pCur, str, _Strlen(str)) == 0)
		{
			return TRUE;
		}

		LexNextLine(fd);
	}

	return FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void LexNextLine(u16 fd)
{
	ST_LEX* p = &Lex[fd];

	while(*p->pCur != '\0')
	{
		if(*p->pCur == CR)
		{
			p->pCur += 2;
			return;
		}

		if(*p->pCur == LF)
		{
			p->pCur += 1;
			return;
		}

		p->pCur++;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void LexUnGet(u16 fd)
{
	Lex[fd].pOld = Lex[fd].pCur;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool LexIsEnd(u16 fd)
{
	return (Lex[fd].pCur == '\0') ? TRUE : FALSE;
}
