#include "state.h"
#include "bg.h"
#include "spr.h"
#include "debug.h"
#include "common.h"
#include "intr_arm.h"
#include "key.h"
#include "pcg.h"
#include "dma_arm.h"
#include "msg.h"
#include "lex.h"
#include "fade.h"
#include "hero.h"
#include "game.h"


//---------------------------------------------------------------------------
u16 State;

// lib/common.c
extern u32 dummy;

//---------------------------------------------------------------------------
EWRAM_CODE void StateInit()
{
	REG_WSCNT = 0x4317;
	_Srand(0x1234567);

	State = STATE_INIT;
}
//---------------------------------------------------------------------------
IWRAM_CODE void StateExec()
{
	for(;;)
	{
		switch(State)
		{
		case STATE_INIT:    StateExecInit();    break;
		case STATE_RESET:   StateExecReset();   break;
		case STATE_RUN:     StateExecRun();     break;
		case STATE_RESTART: StateExecRestart(); break;

		default:
			SystemError("[Err] StateMain State");
			break;
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void StateExecInit()
{
	MyMemInit();
	GbfsInit();

	BgInit();
	SprInit();
	KeyInit();
	PcgInit();
	DmaInit();

	IntrInit();
	IntrStart();

	State = STATE_RESET;
}
//---------------------------------------------------------------------------
IWRAM_CODE void StateExecReset()
{
	DebugInit();
	FadeInit();
	LexInit();

	GameInit();

	State = STATE_RUN;
}
//---------------------------------------------------------------------------
IWRAM_CODE void StateExecRun()
{
	while(GameIsOver() == FALSE)
	{
		SystemCall(5);

		// 
		SprClean();
		PcgClean();

		// Draw
		GameDraw();
		DebugDraw();
		DmaTransfer();

		// Exec
		KeyExec();
		GameExec();
		DebugExec();
	}

	State = STATE_RESTART;
}
//---------------------------------------------------------------------------
IWRAM_CODE void StateExecRestart()
{
	FadeIn();

	StateExecInit();
	StateExecReset();

	GameDraw();
	DebugDraw();

	FadeOut();

	State = STATE_RUN;
}
